Class: CapistranoMulticonfigParallel::CLI

Inherits:
Object
  • Object
show all
Extended by:
ApplicationHelper
Defined in:
lib/capistrano_multiconfig_parallel/cli.rb

Overview

this is the class that will be invoked from terminal , and willl use the invoke task as the primary function.

Class Method Summary collapse

Methods included from ApplicationHelper

parse_task_string, strip_characters_from_string

Methods included from StagesHelper

check_stage_path, checks_paths, fetch_stages, fetch_stages_paths, stages_paths

Methods included from CoreHelper

app_configuration, app_debug_enabled?, app_logger, ask_confirm, check_terminal_tty, debug_websocket?, execute_with_rescue, find_config_type, find_loaded_gem, find_worker_log, force_confirmation, format_error, log_error, log_to_file, rescue_error, rescue_interrupt, show_warning, websocket_config, websocket_server_config

Methods included from InternalHelper

config_file, custom_commands, default_internal_config, detect_root, enable_main_log_file, find_env_multi_cap_root, internal_config_directory, internal_config_file, log_directory, main_log_file, root, try_detect_capfile

Class Method Details

.start ⇒ Object

method used to start



10
11
12
13
14
15
16
17
18
19
20
# File 'lib/capistrano_multiconfig_parallel/cli.rb', line 10

def start
  execute_with_rescue('stderr') do
    verify_validation
    job_manager = CapistranoMulticonfigParallel::Application.new
    if job_manager.argv[CapistranoMulticonfigParallel::ENV_KEY_JOB_ID].blank?
      job_manager.start
    else
      Capistrano::Application.new.run
    end
  end
end

.verify_validation ⇒ Object



22
23
24
25
26
# File 'lib/capistrano_multiconfig_parallel/cli.rb', line 22

def verify_validation
  check_terminal_tty
  CapistranoMulticonfigParallel.original_args = ARGV.dup
  CapistranoMulticonfigParallel.configuration_valid?
end