Class: CapistranoMulticonfigParallel::CelluloidWorker

Inherits:
Object
  • Object
show all
Includes:
ApplicationHelper, Celluloid, Celluloid::Logger, Celluloid::Notifications
Defined in:
lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb

Overview

rubocop:disable ClassLength worker that will spawn a child process in order to execute a capistrano job and monitor that process

Defined Under Namespace

Classes: TaskFailed

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from ApplicationHelper

parse_task_string, strip_characters_from_string

Methods included from StagesHelper

check_stage_path, checks_paths, fetch_stages, fetch_stages_paths, sorted_paths, stages_paths

Methods included from CoreHelper

app_configuration, app_debug_enabled?, app_logger, ask_confirm, check_terminal_tty, debug_websocket?, execute_with_rescue, find_config_type, find_loaded_gem, find_worker_log, force_confirmation, format_error, log_error, log_to_file, rescue_error, rescue_interrupt, show_warning, websocket_config, websocket_server_config

Methods included from InternalHelper

config_file, custom_commands, default_internal_config, detect_root, enable_main_log_file, find_env_multi_cap_root, internal_config_directory, internal_config_file, log_directory, main_log_file, root, try_detect_capfile

Instance Attribute Details

#action_name ⇒ Object

Returns the value of attribute action_name.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def action_name
  @action_name
end

#app_name ⇒ Object

Returns the value of attribute app_name.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def app_name
  @app_name
end

#client ⇒ Object

Returns the value of attribute client.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def client
  @client
end

#current_task_number ⇒ Object

Returns the value of attribute current_task_number.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def current_task_number
  @current_task_number
end

#env_name ⇒ Object

Returns the value of attribute env_name.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def env_name
  @env_name
end

#env_options ⇒ Object

Returns the value of attribute env_options.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def env_options
  @env_options
end

#exit_status ⇒ Object

Returns the value of attribute exit_status.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def exit_status
  @exit_status
end

#filename ⇒ Object

Returns the value of attribute filename.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def filename
  @filename
end

#invocation_chain ⇒ Object

Returns the value of attribute invocation_chain.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def invocation_chain
  @invocation_chain
end

#job ⇒ Hash

Returns options used for executing capistrano task.

Returns:

  • (Hash) —

    options used for executing capistrano task



20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 20

class CelluloidWorker
  include Celluloid
  include Celluloid::Notifications
  include Celluloid::Logger
  include CapistranoMulticonfigParallel::ApplicationHelper
  class TaskFailed < StandardError; end

  attr_accessor :job, :manager, :job_id, :app_name, :env_name, :action_name, :env_options, :machine, :client, :task_argv,
                :rake_tasks, :current_task_number, # tracking tasks
                :successfull_subscription, :subscription_channel, :publisher_channel, # for subscriptions and publishing events
                :job_termination_condition, :worker_state, :invocation_chain, :filename, :worker_log, :exit_status

  def work(job, manager)
    @job = job
    @job_id = job.id
    @worker_state = 'started'
    @manager = manager
    @job_confirmation_conditions = []
    log_to_file("worker #{@job_id} received #{job.inspect}")
    @subscription_channel = "worker_#{@job_id}"
    @machine = CapistranoMulticonfigParallel::StateMachine.new(job, Actor.current)
    manager.register_worker_for_job(job, Actor.current)
  end

  def start_task
    @manager.setup_worker_conditions(@job)
    log_to_file("exec worker #{@job_id} starts task with #{@job.inspect}")
    @client = CelluloidPubsub::Client.connect(actor: Actor.current, enable_debug: debug_websocket?, channel: subscription_channel)
  end

  def publish_rake_event(data)
    @client.publish(rake_actor_id(data), data)
  end

  def rake_actor_id(_data)
    "rake_worker_#{@job_id}"
  end

  def on_message(message)
    log_to_file("worker #{@job_id} received:  #{message.inspect}")
    if @client.succesfull_subscription?(message)
      @successfull_subscription = true
      execute_after_succesfull_subscription
    else
      handle_subscription(message)
    end
  end

  def execute_after_succesfull_subscription
    async.execute_deploy
  end

  def rake_tasks
    @rake_tasks ||= []
  end

  def invocation_chain
    @invocation_chain ||= []
  end

  def execute_deploy
    log_to_file("invocation chain #{@job_id} is : #{@rake_tasks.inspect}")
    check_child_proces
    log_to_file("worker #{@job_id} executes: #{@job.build_capistrano_task}")
    @child_process.async.work(@job, @job.build_capistrano_task, actor: Actor.current, silent: true)
    @manager.wait_task_confirmations_worker(@job)
  end

  def check_child_proces
    if !defined?(@child_process) || @child_process.nil?
      @child_process = CapistranoMulticonfigParallel::ChildProcess.new
      Actor.current.link @child_process
    else
      @client.unsubscribe("rake_worker_#{@job_id}_count")
      @child_process.exit_status = nil
    end
  end

  def on_close(code, reason)
    log_to_file("worker #{@job_id} websocket connection closed: #{code.inspect}, #{reason.inspect}")
  end

  def check_gitflow
    return if @job.stage != 'staging' || !@manager.can_tag_staging? || !executed_task?(CapistranoMulticonfigParallel::GITFLOW_TAG_STAGING_TASK)
    @manager.dispatch_new_job(@job, 'env' => 'production')
  end

  def handle_subscription(message)
    if message_is_about_a_task?(message)
      check_gitflow
      save_tasks_to_be_executed(message)
      async.update_machine_state(message['task']) # if message['action'] == 'invoke'
      log_to_file("worker #{@job_id} state is #{@machine.state}")
      task_approval(message)
    elsif message_is_for_stdout?(message)
      result = Celluloid::Actor[:terminal_server].show_confirmation(message['question'], message['default'])
      publish_rake_event(message.merge('action' => 'stdin', 'result' => result, 'client_action' => 'stdin'))
    else
      log_to_file(message, @job_id)
    end
  end

  def message_is_for_stdout?(message)
    message.present? && message.is_a?(Hash) && message['action'].present? && message['job_id'].present? && message['action'] == 'stdout'
  end

  def message_is_about_a_task?(message)
    message.present? && message.is_a?(Hash) && message['action'].present? && message['job_id'].present? && message['task'].present?
  end

  def executed_task?(task)
    rake_tasks.present? && rake_tasks.index(task.to_s).present?
  end

  def task_approval(message)
    job_conditions = @manager.job_to_condition[@job_id]
    if job_conditions.present? && app_configuration.task_confirmations.include?(message['task']) && message['action'] == 'invoke'
      task_confirmation = job_conditions[message['task']]
      task_confirmation[:status] = 'confirmed'
      task_confirmation[:condition].signal(message['task'])
    else
      publish_rake_event(message.merge('approved' => 'yes'))
    end
  end

  def save_tasks_to_be_executed(message)
    log_to_file("worler #{@job_id} current invocation chain : #{rake_tasks.inspect}")
    rake_tasks << message['task'] if rake_tasks.last != message['task']
    invocation_chain << message['task'] if invocation_chain.last != message['task']
  end

  def update_machine_state(name)
    log_to_file("worker #{@job_id} triest to transition from #{@machine.state} to  #{name}")
    @machine.transitions.on(name.to_s, @machine.state => name.to_s)
    @machine.go_to_transition(name.to_s)
    abort(CapistranoMulticonfigParallel::CelluloidWorker::TaskFailed.new("task #{@action} failed ")) if name == 'deploy:failed' # force worker to rollback
  end

  def send_msg(channel, message = nil)
    publish channel, message.present? && message.is_a?(Hash) ? { job_id: @job_id }.merge(message) : { job_id: @job_id, time: Time.now }
  end

  def finish_worker
    @manager.mark_completed_remaining_tasks(@job)
    @job.status = 'finished'
    @manager.workers_terminated.signal('completed') if @manager.alive? && @manager.all_workers_finished?
  end

  def notify_finished(exit_status)
    if exit_status != 0
      log_to_file("worker #{job_id} tries to terminate")
      abort(CapistranoMulticonfigParallel::CelluloidWorker::TaskFailed.new("task  failed with exit status #{exit_status.inspect} ")) # force worker to rollback
    else
      async.update_machine_state('FINISHED')
      log_to_file("worker #{job_id} notifies manager has finished")
      async.finish_worker
    end
  end
end

#job_id ⇒ Object

Returns the value of attribute job_id.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def job_id
  @job_id
end

#job_termination_condition ⇒ Object

Returns the value of attribute job_termination_condition.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def job_termination_condition
  @job_termination_condition
end

#machine ⇒ Object

Returns the value of attribute machine.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def machine
  @machine
end

#manager ⇒ CapistranoMulticonfigParallel::CelluloidManager

Returns the instance of the manager that delegated the job to this worker.

Returns:



20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 20

class CelluloidWorker
  include Celluloid
  include Celluloid::Notifications
  include Celluloid::Logger
  include CapistranoMulticonfigParallel::ApplicationHelper
  class TaskFailed < StandardError; end

  attr_accessor :job, :manager, :job_id, :app_name, :env_name, :action_name, :env_options, :machine, :client, :task_argv,
                :rake_tasks, :current_task_number, # tracking tasks
                :successfull_subscription, :subscription_channel, :publisher_channel, # for subscriptions and publishing events
                :job_termination_condition, :worker_state, :invocation_chain, :filename, :worker_log, :exit_status

  def work(job, manager)
    @job = job
    @job_id = job.id
    @worker_state = 'started'
    @manager = manager
    @job_confirmation_conditions = []
    log_to_file("worker #{@job_id} received #{job.inspect}")
    @subscription_channel = "worker_#{@job_id}"
    @machine = CapistranoMulticonfigParallel::StateMachine.new(job, Actor.current)
    manager.register_worker_for_job(job, Actor.current)
  end

  def start_task
    @manager.setup_worker_conditions(@job)
    log_to_file("exec worker #{@job_id} starts task with #{@job.inspect}")
    @client = CelluloidPubsub::Client.connect(actor: Actor.current, enable_debug: debug_websocket?, channel: subscription_channel)
  end

  def publish_rake_event(data)
    @client.publish(rake_actor_id(data), data)
  end

  def rake_actor_id(_data)
    "rake_worker_#{@job_id}"
  end

  def on_message(message)
    log_to_file("worker #{@job_id} received:  #{message.inspect}")
    if @client.succesfull_subscription?(message)
      @successfull_subscription = true
      execute_after_succesfull_subscription
    else
      handle_subscription(message)
    end
  end

  def execute_after_succesfull_subscription
    async.execute_deploy
  end

  def rake_tasks
    @rake_tasks ||= []
  end

  def invocation_chain
    @invocation_chain ||= []
  end

  def execute_deploy
    log_to_file("invocation chain #{@job_id} is : #{@rake_tasks.inspect}")
    check_child_proces
    log_to_file("worker #{@job_id} executes: #{@job.build_capistrano_task}")
    @child_process.async.work(@job, @job.build_capistrano_task, actor: Actor.current, silent: true)
    @manager.wait_task_confirmations_worker(@job)
  end

  def check_child_proces
    if !defined?(@child_process) || @child_process.nil?
      @child_process = CapistranoMulticonfigParallel::ChildProcess.new
      Actor.current.link @child_process
    else
      @client.unsubscribe("rake_worker_#{@job_id}_count")
      @child_process.exit_status = nil
    end
  end

  def on_close(code, reason)
    log_to_file("worker #{@job_id} websocket connection closed: #{code.inspect}, #{reason.inspect}")
  end

  def check_gitflow
    return if @job.stage != 'staging' || !@manager.can_tag_staging? || !executed_task?(CapistranoMulticonfigParallel::GITFLOW_TAG_STAGING_TASK)
    @manager.dispatch_new_job(@job, 'env' => 'production')
  end

  def handle_subscription(message)
    if message_is_about_a_task?(message)
      check_gitflow
      save_tasks_to_be_executed(message)
      async.update_machine_state(message['task']) # if message['action'] == 'invoke'
      log_to_file("worker #{@job_id} state is #{@machine.state}")
      task_approval(message)
    elsif message_is_for_stdout?(message)
      result = Celluloid::Actor[:terminal_server].show_confirmation(message['question'], message['default'])
      publish_rake_event(message.merge('action' => 'stdin', 'result' => result, 'client_action' => 'stdin'))
    else
      log_to_file(message, @job_id)
    end
  end

  def message_is_for_stdout?(message)
    message.present? && message.is_a?(Hash) && message['action'].present? && message['job_id'].present? && message['action'] == 'stdout'
  end

  def message_is_about_a_task?(message)
    message.present? && message.is_a?(Hash) && message['action'].present? && message['job_id'].present? && message['task'].present?
  end

  def executed_task?(task)
    rake_tasks.present? && rake_tasks.index(task.to_s).present?
  end

  def task_approval(message)
    job_conditions = @manager.job_to_condition[@job_id]
    if job_conditions.present? && app_configuration.task_confirmations.include?(message['task']) && message['action'] == 'invoke'
      task_confirmation = job_conditions[message['task']]
      task_confirmation[:status] = 'confirmed'
      task_confirmation[:condition].signal(message['task'])
    else
      publish_rake_event(message.merge('approved' => 'yes'))
    end
  end

  def save_tasks_to_be_executed(message)
    log_to_file("worler #{@job_id} current invocation chain : #{rake_tasks.inspect}")
    rake_tasks << message['task'] if rake_tasks.last != message['task']
    invocation_chain << message['task'] if invocation_chain.last != message['task']
  end

  def update_machine_state(name)
    log_to_file("worker #{@job_id} triest to transition from #{@machine.state} to  #{name}")
    @machine.transitions.on(name.to_s, @machine.state => name.to_s)
    @machine.go_to_transition(name.to_s)
    abort(CapistranoMulticonfigParallel::CelluloidWorker::TaskFailed.new("task #{@action} failed ")) if name == 'deploy:failed' # force worker to rollback
  end

  def send_msg(channel, message = nil)
    publish channel, message.present? && message.is_a?(Hash) ? { job_id: @job_id }.merge(message) : { job_id: @job_id, time: Time.now }
  end

  def finish_worker
    @manager.mark_completed_remaining_tasks(@job)
    @job.status = 'finished'
    @manager.workers_terminated.signal('completed') if @manager.alive? && @manager.all_workers_finished?
  end

  def notify_finished(exit_status)
    if exit_status != 0
      log_to_file("worker #{job_id} tries to terminate")
      abort(CapistranoMulticonfigParallel::CelluloidWorker::TaskFailed.new("task  failed with exit status #{exit_status.inspect} ")) # force worker to rollback
    else
      async.update_machine_state('FINISHED')
      log_to_file("worker #{job_id} notifies manager has finished")
      async.finish_worker
    end
  end
end

#publisher_channel ⇒ Object

Returns the value of attribute publisher_channel.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def publisher_channel
  @publisher_channel
end

#rake_tasks ⇒ Object

Returns the value of attribute rake_tasks.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def rake_tasks
  @rake_tasks
end

#subscription_channel ⇒ Object

Returns the value of attribute subscription_channel.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def subscription_channel
  @subscription_channel
end

#successfull_subscription ⇒ Object

Returns the value of attribute successfull_subscription.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def successfull_subscription
  @successfull_subscription
end

#task_argv ⇒ Object

Returns the value of attribute task_argv.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def task_argv
  @task_argv
end

#worker_log ⇒ Object

Returns the value of attribute worker_log.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def worker_log
  @worker_log
end

#worker_state ⇒ Object

Returns the value of attribute worker_state.



27
28
29
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 27

def worker_state
  @worker_state
end

Instance Method Details

#check_child_proces ⇒ Object



88
89
90
91
92
93
94
95
96
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 88

def check_child_proces
  if !defined?(@child_process) || @child_process.nil?
    @child_process = CapistranoMulticonfigParallel::ChildProcess.new
    Actor.current.link @child_process
  else
    @client.unsubscribe("rake_worker_#{@job_id}_count")
    @child_process.exit_status = nil
  end
end

#check_gitflow ⇒ Object



102
103
104
105
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 102

def check_gitflow
  return if @job.stage != 'staging' || !@manager.can_tag_staging? || !executed_task?(CapistranoMulticonfigParallel::GITFLOW_TAG_STAGING_TASK)
  @manager.dispatch_new_job(@job, 'env' => 'production')
end

#execute_after_succesfull_subscription ⇒ Object



68
69
70
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 68

def execute_after_succesfull_subscription
  async.execute_deploy
end

#execute_deploy ⇒ Object



80
81
82
83
84
85
86
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 80

def execute_deploy
  log_to_file("invocation chain #{@job_id} is : #{@rake_tasks.inspect}")
  check_child_proces
  log_to_file("worker #{@job_id} executes: #{@job.build_capistrano_task}")
  @child_process.async.work(@job, @job.build_capistrano_task, actor: Actor.current, silent: true)
  @manager.wait_task_confirmations_worker(@job)
end

#executed_task?(task) ⇒ Boolean

Returns:

  • (Boolean)


130
131
132
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 130

def executed_task?(task)
  rake_tasks.present? && rake_tasks.index(task.to_s).present?
end

#finish_worker ⇒ Object



162
163
164
165
166
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 162

def finish_worker
  @manager.mark_completed_remaining_tasks(@job)
  @job.status = 'finished'
  @manager.workers_terminated.signal('completed') if @manager.alive? && @manager.all_workers_finished?
end

#handle_subscription(message) ⇒ Object



107
108
109
110
111
112
113
114
115
116
117
118
119
120
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 107

def handle_subscription(message)
  if message_is_about_a_task?(message)
    check_gitflow
    save_tasks_to_be_executed(message)
    async.update_machine_state(message['task']) # if message['action'] == 'invoke'
    log_to_file("worker #{@job_id} state is #{@machine.state}")
    task_approval(message)
  elsif message_is_for_stdout?(message)
    result = Celluloid::Actor[:terminal_server].show_confirmation(message['question'], message['default'])
    publish_rake_event(message.merge('action' => 'stdin', 'result' => result, 'client_action' => 'stdin'))
  else
    log_to_file(message, @job_id)
  end
end

#message_is_about_a_task?(message) ⇒ Boolean

Returns:

  • (Boolean)


126
127
128
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 126

def message_is_about_a_task?(message)
  message.present? && message.is_a?(Hash) && message['action'].present? && message['job_id'].present? && message['task'].present?
end

#message_is_for_stdout?(message) ⇒ Boolean

Returns:

  • (Boolean)


122
123
124
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 122

def message_is_for_stdout?(message)
  message.present? && message.is_a?(Hash) && message['action'].present? && message['job_id'].present? && message['action'] == 'stdout'
end

#notify_finished(exit_status) ⇒ Object



168
169
170
171
172
173
174
175
176
177
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 168

def notify_finished(exit_status)
  if exit_status != 0
    log_to_file("worker #{job_id} tries to terminate")
    abort(CapistranoMulticonfigParallel::CelluloidWorker::TaskFailed.new("task  failed with exit status #{exit_status.inspect} ")) # force worker to rollback
  else
    async.update_machine_state('FINISHED')
    log_to_file("worker #{job_id} notifies manager has finished")
    async.finish_worker
  end
end

#on_close(code, reason) ⇒ Object



98
99
100
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 98

def on_close(code, reason)
  log_to_file("worker #{@job_id} websocket connection closed: #{code.inspect}, #{reason.inspect}")
end

#on_message(message) ⇒ Object



58
59
60
61
62
63
64
65
66
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 58

def on_message(message)
  log_to_file("worker #{@job_id} received:  #{message.inspect}")
  if @client.succesfull_subscription?(message)
    @successfull_subscription = true
    execute_after_succesfull_subscription
  else
    handle_subscription(message)
  end
end

#publish_rake_event(data) ⇒ Object



50
51
52
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 50

def publish_rake_event(data)
  @client.publish(rake_actor_id(data), data)
end

#rake_actor_id(_data) ⇒ Object



54
55
56
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 54

def rake_actor_id(_data)
  "rake_worker_#{@job_id}"
end

#save_tasks_to_be_executed(message) ⇒ Object



145
146
147
148
149
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 145

def save_tasks_to_be_executed(message)
  log_to_file("worler #{@job_id} current invocation chain : #{rake_tasks.inspect}")
  rake_tasks << message['task'] if rake_tasks.last != message['task']
  invocation_chain << message['task'] if invocation_chain.last != message['task']
end

#send_msg(channel, message = nil) ⇒ Object



158
159
160
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 158

def send_msg(channel, message = nil)
  publish channel, message.present? && message.is_a?(Hash) ? { job_id: @job_id }.merge(message) : { job_id: @job_id, time: Time.now }
end

#start_task ⇒ Object



44
45
46
47
48
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 44

def start_task
  @manager.setup_worker_conditions(@job)
  log_to_file("exec worker #{@job_id} starts task with #{@job.inspect}")
  @client = CelluloidPubsub::Client.connect(actor: Actor.current, enable_debug: debug_websocket?, channel: subscription_channel)
end

#task_approval(message) ⇒ Object



134
135
136
137
138
139
140
141
142
143
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 134

def task_approval(message)
  job_conditions = @manager.job_to_condition[@job_id]
  if job_conditions.present? && app_configuration.task_confirmations.include?(message['task']) && message['action'] == 'invoke'
    task_confirmation = job_conditions[message['task']]
    task_confirmation[:status] = 'confirmed'
    task_confirmation[:condition].signal(message['task'])
  else
    publish_rake_event(message.merge('approved' => 'yes'))
  end
end

#update_machine_state(name) ⇒ Object



151
152
153
154
155
156
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 151

def update_machine_state(name)
  log_to_file("worker #{@job_id} triest to transition from #{@machine.state} to  #{name}")
  @machine.transitions.on(name.to_s, @machine.state => name.to_s)
  @machine.go_to_transition(name.to_s)
  abort(CapistranoMulticonfigParallel::CelluloidWorker::TaskFailed.new("task #{@action} failed ")) if name == 'deploy:failed' # force worker to rollback
end

#work(job, manager) ⇒ Object



32
33
34
35
36
37
38
39
40
41
42
# File 'lib/capistrano_multiconfig_parallel/celluloid/celluloid_worker.rb', line 32

def work(job, manager)
  @job = job
  @job_id = job.id
  @worker_state = 'started'
  @manager = manager
  @job_confirmation_conditions = []
  log_to_file("worker #{@job_id} received #{job.inspect}")
  @subscription_channel = "worker_#{@job_id}"
  @machine = CapistranoMulticonfigParallel::StateMachine.new(job, Actor.current)
  manager.register_worker_for_job(job, Actor.current)
end