Class: CapistranoMulticonfigParallel::CLI

Inherits:
Object
  • Object
show all
Defined in:
lib/capistrano_multiconfig_parallel/cli.rb

Overview

this is the class that will be invoked from terminal , and willl use the invoke task as the primary function.

Class Method Summary collapse

Class Method Details

.start ⇒ Object



6
7
8
9
10
11
12
13
14
15
16
# File 'lib/capistrano_multiconfig_parallel/cli.rb', line 6

def self.start
  CapistranoMulticonfigParallel.check_terminal_tty
  CapistranoMulticonfigParallel.original_args = ARGV.dup
  CapistranoMulticonfigParallel::Application.new.run
   rescue Interrupt
 `stty icanon echo`
 $stderr.puts 'Command cancelled.'
   rescue => error
 $stderr.puts error
 exit(1)
end